#ifdef _WIN32
#include <windows.h>
#include <VideoIM.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#ifndef __APPLE__
#include <GL/gl.h>
#include <GL/glut.h>
#else
#include <OpenGL/gl.h>
#include <GLUT/glut.h>
#endif
#include <AR/gsub.h>
#include <AR/video.h>
#include <AR/param.h>
#include <AR/ar.h>

//
// Camera configuration.
//
#ifdef _WIN32
char			*vconf = "Data\\WDM_camera_flipV.xml";
#else
char			*vconf = "";
#endif

int             xsize, ysize;
int             thresh = 100;
int             count = 0;


char           *cparam_name    = "Data/camera_para.dat";
ARParam         cparam;


char            *NomesDePadroes[] = { "Data/multi/patt.a",
                                      "Data/patt.Kanji",
                                      "Data/multi/patt.b",
                                      "Data/multi/patt.c",
                                      "Data/multi/patt.d",
                                      "Data/multi/patt.f",
                                      "Data/multi/patt.g",
                                      "Data/patt.Hiro"
                                    };

int qtdDeMarcadores = sizeof(NomesDePadroes)/sizeof(char*);
int EstaVisivel[10];
int IDsDePadroes[10];

double          patt_width     = 30.0;
double          patt_center[2] = {0.0, 0.0};
double          patt_trans[3][4];

typedef struct Ponto
{
    GLfloat x,y,z;

} Ponto;

Ponto PosicoesDasEtiquetas[10];

static void   init(void);
static void   cleanup(void);
static void   keyEvent( unsigned char key, int x, int y);
static void   mainLoop(void);
static void   draw( int );

void setColor(int c)
{
    switch(c)
    {
    case 0:
        glColor3f(1,0,0);
        break;
    case 1:
        glColor3f(0,1,0);
        break;
    case 2:
        glColor3f(0,0,1);
        break;
    case 3:
        glColor3f(1,1,0);
        break; // amarelo
    case 4:
        glColor3f(0,1,1);
        break; //
    case 5:
        glColor3f(1,0.3,0.3);
        break;
    case 6:
        glColor3f(1,0.5,0.8);
        break;
    case 7:
        glColor3f(0.9,0.5,0.2);
        break;
    }
}
void ImprimePonto(Ponto P)
{
    printf("%7.3f, %7.3f, %7.3f", P.x, P.y, P.z);
}
// ****************************************************
//
// ****************************************************
Ponto ObtemCentroDaEtiqueta()
{
    GLfloat matriz_gl[4][4];
    Ponto temp;

    glGetFloatv(GL_MODELVIEW_MATRIX,&matriz_gl[0][0]);

    temp.x = matriz_gl[3][0];
    temp.y = matriz_gl[3][1];
    temp.z = matriz_gl[3][2];
    return temp;
}


static void keyEvent( unsigned char key, int x, int y)
{
    /* quit if the ESC key is pressed */
    if( key == 0x1b )
    {
        printf("*** %f (frame/sec)\n", (double)count/arUtilTimer());
        cleanup();
        exit(0);
    }
}

void DefineLuz()
{
    // Define cores para um objeto dourado
    GLfloat LuzAmbiente[]   = {0.4, 0.4, 0.4f } ;
    GLfloat LuzDifusa[]   = {0.7, 0.7, 0.7};
    GLfloat LuzEspecular[] = {0.9f, 0.9f, 0.9 };
    GLfloat PosicaoLuz0[]  = {3.0f, 3.0f, 0.0f };
    GLfloat Especularidade[] = {1.0f, 1.0f, 1.0f};

    // ****************  Fonte de Luz 0

    glEnable ( GL_COLOR_MATERIAL );

    // Habilita o uso de iluminao
    glEnable(GL_LIGHTING);

    // Ativa o uso da luz ambiente
    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, LuzAmbiente);
    // Define os parametros da luz nmero Zero
    glLightfv(GL_LIGHT0, GL_AMBIENT, LuzAmbiente);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, LuzDifusa  );
    glLightfv(GL_LIGHT0, GL_SPECULAR, LuzEspecular  );
    glLightfv(GL_LIGHT0, GL_POSITION, PosicaoLuz0 );
    glEnable(GL_LIGHT0);

    // Ativa o "Color Tracking"
    glEnable(GL_COLOR_MATERIAL);

    // Define a reflectancia do material
    glMaterialfv(GL_FRONT,GL_SPECULAR, Especularidade);

    // Define a concentraoo do brilho.
    // Quanto maior o valor do Segundo parametro, mais
    // concentrado ser o brilho. (Valores vlidos: de 0 a 128)
    glMateriali(GL_FRONT,GL_SHININESS,51);
    glClearDepth( 1.0 );
    glClear(GL_DEPTH_BUFFER_BIT);
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LEQUAL);
}

// ****************************************************
//
// ****************************************************
int BuscaMarcador(int nroDaEtiqueta, ARMarkerInfo marker_info[], int marker_num)
{
    int j, k;
    k = -1;
    // printf("Buscando %d - %s.", nroDaEtiqueta, NomesDePadroes[nroDaEtiqueta]);
    int id = IDsDePadroes[nroDaEtiqueta];

    for( j = 0; j < marker_num; j++ )
    {
        if( id == marker_info[j].id )
        {
            if( k == -1 )
                k = j;
            else if( marker_info[k].cf <= marker_info[j].cf )
                k = j;
        }
    }
    // printf("Confianca: %5.3f - ", marker_info[k].cf);
    if (marker_info[k].cf > 0.75)
    {
        // printf("Encontrou %s.\n", NomesDePadroes[marker_info[k].id]);
        return k;
    }
    else
    {
        // printf("Nao encontrou.\n");
        return -1;
    }
}
// *************************************************************
// Desenha uma linha entre um par de marcadores
// *************************************************************
void DesenhaLinha(int i, int j)
{
    glPushMatrix();
    glLoadIdentity();
    glBegin(GL_LINES);
    glVertex3f(PosicoesDasEtiquetas[i].x,PosicoesDasEtiquetas[i].y,PosicoesDasEtiquetas[i].z);
    glVertex3f(PosicoesDasEtiquetas[j].x,PosicoesDasEtiquetas[j].y,PosicoesDasEtiquetas[j].z);
    glEnd();
    glPopMatrix();
}
// *************************************************************
// Desenha uma linha entre cada par de marcadores visveis
// *************************************************************
void DesenhaLinhas()
{
    int i,j;
    glColor3f(1,1,1);
    glLineWidth(3);
    for(i=0; i<qtdDeMarcadores; i++)
    {
        for(j=0; j<qtdDeMarcadores; j++)
            if (i>=j)
                if (EstaVisivel[i] && EstaVisivel[j])
                    DesenhaLinha(i,j);
    }
    glLineWidth(1);
}
/* main loop */
static void mainLoop(void)
{

    ARUint8         *dataPtr;
    ARMarkerInfo    *marker_info;
    int             marker_num;
    int             k, idCorrente,EtiquetaIdentificada;

    //arVideoCapNext();
    /* grab a vide frame */
    if( (dataPtr = (ARUint8 *)arVideoGetImage()) == NULL )
    {
        arUtilSleep(2);
        return;
    }
    if( count == 0 )
        arUtilTimerReset();
    count++;

    argDrawMode2D();
    glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
    argDispImage( dataPtr, 0,0 );

    /* detect the markers in the video frame */
    if( arDetectMarker(dataPtr, thresh, &marker_info, &marker_num) < 0 )
    {
        cleanup();
        exit(0);
    }

    arVideoCapNext();

    argDrawMode3D();
    argDraw3dCamera( 0, 0 );
    DefineLuz();
    printf("\n");
    printf("********\n");
    for(idCorrente=0; idCorrente<qtdDeMarcadores; idCorrente++)
        EstaVisivel[idCorrente] = 0;

    for(idCorrente=0; idCorrente<qtdDeMarcadores; idCorrente++)
    {
        k = BuscaMarcador(idCorrente, marker_info, marker_num);
        EtiquetaIdentificada = marker_info[k].id;
        if( k != -1 )
        {
            printf("Marcador %s encontrado.\n", NomesDePadroes[EtiquetaIdentificada]);
            /* get the transformation between the marker and the real camera */
            arGetTransMat(&marker_info[k], patt_center, patt_width, patt_trans);
            draw(marker_info[k].id);
            EstaVisivel[marker_info[k].id] = 1;
        }
    }
    DesenhaLinhas();
    printf("Etiquetas visveis: ");
    for(idCorrente=0; idCorrente<qtdDeMarcadores; idCorrente++)
        if (EstaVisivel[idCorrente])
            printf("%3d",idCorrente);
    printf(".\n");
    argSwapBuffers();
}

static void init( void )
{
    ARParam  wparam;
    int idCorrente;

    /* open the video path */
    if( arVideoOpen( vconf ) < 0 )
        exit(0);
    /* find the size of the window */
    if( arVideoInqSize(&xsize, &ysize) < 0 )
        exit(0);
    printf("Image size (x,y) = (%d,%d)\n", xsize, ysize);

    /* set the initial camera parameters */
    if( arParamLoad(cparam_name, 1, &wparam) < 0 )
    {
        printf("Camera parameter load error !!\n");
        getchar();
        exit(0);
    }
    arParamChangeSize( &wparam, xsize, ysize, &cparam );
    arInitCparam( &cparam );
    printf("*** Camera Parameter ***\n");
    arParamDisp( &cparam );

    printf("Qtd de Marcadores: %d -> ", qtdDeMarcadores);
    for(idCorrente=0; idCorrente<qtdDeMarcadores; idCorrente++)
    {
        IDsDePadroes[idCorrente] = arLoadPatt(NomesDePadroes[idCorrente]);
        printf("%3d", IDsDePadroes[idCorrente]);
        if (IDsDePadroes[idCorrente] <0)
        {
            printf("Arquivo de padrao inexistente !!\n");
            getchar();
            exit(0);
        }
    }
    printf(".\n");

    /* open the graphics window */
    argInit( &cparam, 1.0, 0, 0, 0, 0 );
}

/* cleanup function called when program exits */
static void cleanup(void)
{
    arVideoCapStop();
    arVideoClose();
    argCleanup();
}

static void draw(int IdDoMarcador)
{

    double gl_para[16];

    /* load the camera transformation matrix */
    argConvGlpara(patt_trans, gl_para);
    glMatrixMode(GL_MODELVIEW);
    glLoadMatrixd( gl_para );

    // Guarda a posicao central da estiqueta
    PosicoesDasEtiquetas[IdDoMarcador] = ObtemCentroDaEtiqueta();

    glPushMatrix();
    setColor(IdDoMarcador);
    glTranslatef( 0.0, 0.0, 7.5 );
    glutSolidCube(15.0);
    glPopMatrix();
}
int main(int argc, char **argv)
{
    glutInit(&argc, argv);
    init();

    arVideoCapStart();
    argMainLoop( NULL, keyEvent, mainLoop );
    return (0);
}
